package com.aucma.production.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.aucma.common.utils.DateUtils;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.aucma.common.annotation.Log;
import com.aucma.common.core.controller.BaseController;
import com.aucma.common.core.domain.AjaxResult;
import com.aucma.common.enums.BusinessType;
import com.aucma.production.domain.BaseBomInfo;
import com.aucma.production.service.IBaseBomInfoService;
import com.aucma.common.utils.poi.ExcelUtil;
import com.aucma.common.core.page.TableDataInfo;

/**
 * 生产BOMController
 *
 * @author Yinq
 * @date 2023-09-28
 */
@RestController
@RequestMapping("/production/baseBomInfo" )
public class BaseBomInfoController extends BaseController {
    @Autowired
    private IBaseBomInfoService baseBomInfoService;

    /**
     * 查询生产BOM列表
     */
    @PreAuthorize("@ss.hasPermi('production:baseBomInfo:list')" )
    @GetMapping("/list" )
    public TableDataInfo list(BaseBomInfo baseBomInfo) {
        startPage();
        List<BaseBomInfo> list = baseBomInfoService.selectBaseBomInfoList(baseBomInfo);
        return getDataTable(list);
    }

    /**
     * 查询生产子BOM树状列表
     */
    @GetMapping("/treeList" )
    public AjaxResult treeList(BaseBomInfo baseBomInfo) {
        List<BaseBomInfo> list = baseBomInfoService.selectBaseBomInfoList(baseBomInfo);
        return success(list);
    }

    /**
     * 导出生产BOM列表
     */
    @PreAuthorize("@ss.hasPermi('production:baseBomInfo:export')" )
    @Log(title = "生产BOM" , businessType = BusinessType.EXPORT)
    @PostMapping("/export" )
    public void export(HttpServletResponse response, BaseBomInfo baseBomInfo) {
        List<BaseBomInfo> list = baseBomInfoService.selectBaseBomInfoList(baseBomInfo);
        ExcelUtil<BaseBomInfo> util = new ExcelUtil<BaseBomInfo>(BaseBomInfo. class);
        util.exportExcel(response, list, "生产BOM数据" );
    }

    /**
     * 获取生产BOM详细信息
     */
    @PreAuthorize("@ss.hasPermi('production:baseBomInfo:query')" )
    @GetMapping(value = "/{objId}" )
    public AjaxResult getInfo(@PathVariable("objId" ) Long objId) {
        return success(baseBomInfoService.selectBaseBomInfoByObjId(objId));
    }

    /**
     * 新增生产BOM
     */
    @PreAuthorize("@ss.hasPermi('production:baseBomInfo:add')" )
    @Log(title = "生产BOM" , businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BaseBomInfo baseBomInfo) {
        baseBomInfo.setCreatedBy(getUsername());
        baseBomInfo.setCreatedTime(DateUtils.getNowDate());
        return toAjax(baseBomInfoService.insertBaseBomInfo(baseBomInfo));
    }

    /**
     * 修改生产BOM
     */
    @PreAuthorize("@ss.hasPermi('production:baseBomInfo:edit')" )
    @Log(title = "生产BOM" , businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BaseBomInfo baseBomInfo) {
        baseBomInfo.setUpdatedBy(getUsername());
        baseBomInfo.setUpdatedTime(DateUtils.getNowDate());
        return toAjax(baseBomInfoService.updateBaseBomInfo(baseBomInfo));
    }

    /**
     * 删除生产BOM
     */
    @PreAuthorize("@ss.hasPermi('production:baseBomInfo:remove')" )
    @Log(title = "生产BOM" , businessType = BusinessType.DELETE)
    @DeleteMapping("/{objIds}" )
    public AjaxResult remove(@PathVariable Long[] objIds) {
        return toAjax(baseBomInfoService.deleteBaseBomInfoByObjIds(objIds));
    }
}
